/*$Id: d_cap.cc,v 16.19 2000/02/03 08:41:50 al Exp $ -*- C++ -*-
 * capacitor models
 * y.x = volts, y.f0 = coulombs, ev = y.f1 = farads
 * m.x = volts, m.c0 = amps,    acg = m.c1 = mhos.
 */
#include "d_cap.h"
/*--------------------------------------------------------------------------*/
bool DEV_CAPACITANCE::do_tr()
{
  initial_condition = NOT_INPUT;
  {if (has_tr_eval()){
    _m0.x = _y0.x = tr_involts_limited();
    tr_eval();
  }else{
    _m0.x = _y0.x = tr_involts();
    assert(_y0.f1 == value());
    _y0.f0 = _y0.x * _y0.f1;
    assert(converged());
  }}
  store_values();
  q_load();
  integrate();
  return converged();
}
/*--------------------------------------------------------------------------*/
double DEV_CAPACITANCE::tr_review()
{
  return review(_m0.c1 * _m0.x + _m0.c0, _mt1.c1 * _mt1.x + _mt1.c0);
}
/*--------------------------------------------------------------------------*/
void DEV_CAPACITANCE::do_ac()
{
  {if (has_ac_eval()){
    ac_eval();
  }else{
    assert(_ev == _y0.f1);
    assert(has_tr_eval() || _ev == value());
  }}
  _acg = _ev * SIM::jomega;
  ac_load();
}
/*--------------------------------------------------------------------------*/
void DEV_CAPACITANCE::integrate()
{
  {if (SIM::mode == sDC  ||  SIM::mode == sOP){
    _m0.c0 = _m0.c1 = 0.;
  }else{
    assert(SIM::mode == sTRAN || SIM::mode == sFOURIER);
    {if (SIM::phase == pINIT_DC){
      {if (_time[0] == 0.){
	{if (!SIM::uic){
	  _m0.c0 = _m0.c1 = 0.;
	}else if (initial_condition != NOT_INPUT){
	  _m0.c1 = 1./OPT::shortckt;
	  _m0.c0 = -initial_condition * _m0.c1;
	}else{
	  untested();
	  _m0.c0 = _m0.c1 = 0.;
	}}
      }else{
	assert(_time[0] > 0);
	/* leave it alone to restart from a previous solution */
      }}
    }else{
      assert(SIM::phase == pTRAN);
      double oldv = _mt1.x;
      double dt = _time[0] - _time[1];
      assert(dt > 0.);
      switch (_method_a){
      case mGEAR:
      case mTRAPGEAR:
      case mTRAPEULER:
      case mEULER:{
	_m0.c1 = _y0.f1 / dt;		    /* first order 	      */
	_m0.c0 = -_m0.c1 * oldv;	    /* (stiff) (backward Euler) */
	break;
      }
      case mTRAP:{			    /* second order 	     */
	double oldi = _mt1.c0 + _mt1.c1*oldv; /* (non-stiff) (trapezoid) */
	_m0.c1 = 2. * _y0.f1 / dt;
	_m0.c0 = -_m0.c1 * oldv - oldi;
	break;
      }
      }
    }}
  }}
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
bool DEV_VCCAP::do_tr()
{
  initial_condition = NOT_INPUT;
  {if (has_tr_eval()){
    untested();
    _m0.x = tr_outvolts();
    _y0.x = tr_involts_limited();
    tr_eval();
    _y0.f1 = _y0.f0;
    _y0.f0 = _m0.x * _y0.f1;
  }else{
    _m0.x = tr_outvolts();
    _y0.x = tr_involts();
    _y0.f1 = value() * _y0.x;
    _y0.f0 = _m0.x * _y0.f1;
    assert(converged());
  }}
  store_values();
  q_load();
  integrate();
  return converged();
}
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
