/*$Id: u_sdp.h,v 16.14 1999/12/19 22:52:34 al Exp $ -*- C++ -*-
 * A class for Size Dependent Parameters, like those used in Spice BSIM models
 * A single read gets the nominal, and length and width dependencies.
 * A single print prints them all.
 * operator() returns the value to use, adjusted for L and W.
 */
#ifndef U_BINPARAM_H
#define U_BINPARAM_H
#include "ap.h"
/*--------------------------------------------------------------------------*/
class SDP {
private:
  double _nom;	// nominal value
  double _ld;	// length dependency
  double _wd;	// width dependency
  double _pd;	// cross-term dependency (p is for product)
public:
  explicit SDP() :_nom(0.), _ld(0.), _wd(0.), _pd(0.) {}
  CS&  get(CS&, const std::string&, AP_MOD=mNONE, double=0);
  void print(OMSTREAM& o, const std::string& name)const;
  double operator()(double L, double W)const {
    return _nom + _ld/L + _wd/W + _pd/(W*L);}
};
/*--------------------------------------------------------------------------*/
/*--------------------------------------------------------------------------*/
#endif
